@@ -13,10 +13,12 @@ use crate::{
     types::{
         self, CreateNamespaceVars, DeployApp, DeployAppConnection, DeployAppVersion,
         DeployAppVersionConnection, DnsDomain, GetAppTemplateFromSlugVariables,
-        GetAppTemplatesVars, GetCurrentUserWithAppsVars, GetDeployAppAndVersion,
-        GetDeployAppVersionsVars, GetNamespaceAppsVars, GetSignedUrlForPackageUploadVariables, Log,
-        LogStream, PackageVersionConnection, PublishDeployAppVars, PushPackageReleasePayload,
-        SignedUrl, TagPackageReleasePayload, UpsertDomainFromZoneFileVars,
+        GetAppTemplatesFromFrameworkVars, GetAppTemplatesFromLanguageVars, GetAppTemplatesVars,
+        GetCurrentUserWithAppsVars, GetDeployAppAndVersion, GetDeployAppVersionsVars,
+        GetNamespaceAppsVars, GetSignedUrlForPackageUploadVariables, GetTemplateFrameworksVars,
+        GetTemplateLanguagesVars, Log, LogStream, PackageVersionConnection, PublishDeployAppVars,
+        PushPackageReleasePayload, SignedUrl, TagPackageReleasePayload,
+        UpsertDomainFromZoneFileVars,
     },
     GraphQLApiFailure, WasmerClient,
 };
@@ -69,6 +71,27 @@ pub async fn fetch_app_template_from_slug(
         .map(|v| v.get_app_template)
 }
 
+/// Fetch app templates.
+pub async fn fetch_app_templates_from_framework(
+    client: &WasmerClient,
+    framework_slug: String,
+    first: i32,
+    after: Option<String>,
+    sort_by: Option<types::AppTemplatesSortBy>,
+) -> Result<Option<types::AppTemplateConnection>, anyhow::Error> {
+    client
+        .run_graphql_strict(types::GetAppTemplatesFromFramework::build(
+            GetAppTemplatesFromFrameworkVars {
+                framework_slug,
+                first,
+                after,
+                sort_by,
+            },
+        ))
+        .await
+        .map(|r| r.get_app_templates)
+}
+
 /// Fetch app templates.
 pub async fn fetch_app_templates(
     client: &WasmerClient,
@@ -145,6 +168,270 @@ pub fn fetch_all_app_templates(
     )
 }
 
+/// Fetch all app templates by paginating through the responses.
+///
+/// Will fetch at most `max` templates.
+pub fn fetch_all_app_templates_from_language(
+    client: &WasmerClient,
+    page_size: i32,
+    sort_by: Option<types::AppTemplatesSortBy>,
+    language: String,
+) -> impl futures::Stream<Item = Result<Vec<types::AppTemplate>, anyhow::Error>> + '_ {
+    let vars = GetAppTemplatesFromLanguageVars {
+        language_slug: language.clone().to_string(),
+        first: page_size,
+        sort_by,
+        after: None,
+    };
+
+    futures::stream::try_unfold(
+        Some(vars),
+        move |vars: Option<types::GetAppTemplatesFromLanguageVars>| async move {
+            let vars = match vars {
+                Some(vars) => vars,
+                None => return Ok(None),
+            };
+
+            let con = client
+                .run_graphql_strict(types::GetAppTemplatesFromLanguage::build(vars.clone()))
+                .await?
+                .get_app_templates
+                .context("backend did not return any data")?;
+
+            let items = con
+                .edges
+                .into_iter()
+                .flatten()
+                .filter_map(|edge| edge.node)
+                .collect::<Vec<_>>();
+
+            let next_cursor = con
+                .page_info
+                .end_cursor
+                .filter(|_| con.page_info.has_next_page);
+
+            let next_vars = next_cursor.map(|after| types::GetAppTemplatesFromLanguageVars {
+                after: Some(after),
+                ..vars
+            });
+
+            #[allow(clippy::type_complexity)]
+            let res: Result<
+                Option<(
+                    Vec<types::AppTemplate>,
+                    Option<types::GetAppTemplatesFromLanguageVars>,
+                )>,
+                anyhow::Error,
+            > = Ok(Some((items, next_vars)));
+
+            res
+        },
+    )
+}
+
+/// Fetch languages from available app templates.
+pub async fn fetch_app_template_languages(
+    client: &WasmerClient,
+    after: Option<String>,
+    first: Option<i32>,
+) -> Result<Option<types::TemplateLanguageConnection>, anyhow::Error> {
+    client
+        .run_graphql_strict(types::GetTemplateLanguages::build(
+            GetTemplateLanguagesVars { after, first },
+        ))
+        .await
+        .map(|r| r.get_template_languages)
+}
+
+/// Fetch all languages from available app templates by paginating through the responses.
+///
+/// Will fetch at most `max` templates.
+pub fn fetch_all_app_template_languages(
+    client: &WasmerClient,
+    page_size: Option<i32>,
+) -> impl futures::Stream<Item = Result<Vec<types::TemplateLanguage>, anyhow::Error>> + '_ {
+    let vars = GetTemplateLanguagesVars {
+        after: None,
+        first: page_size,
+    };
+
+    futures::stream::try_unfold(
+        Some(vars),
+        move |vars: Option<types::GetTemplateLanguagesVars>| async move {
+            let vars = match vars {
+                Some(vars) => vars,
+                None => return Ok(None),
+            };
+
+            let con = client
+                .run_graphql_strict(types::GetTemplateLanguages::build(vars.clone()))
+                .await?
+                .get_template_languages
+                .context("backend did not return any data")?;
+
+            let items = con
+                .edges
+                .into_iter()
+                .flatten()
+                .filter_map(|edge| edge.node)
+                .collect::<Vec<_>>();
+
+            let next_cursor = con
+                .page_info
+                .end_cursor
+                .filter(|_| con.page_info.has_next_page);
+
+            let next_vars = next_cursor.map(|after| types::GetTemplateLanguagesVars {
+                after: Some(after),
+                ..vars
+            });
+
+            #[allow(clippy::type_complexity)]
+            let res: Result<
+                Option<(
+                    Vec<types::TemplateLanguage>,
+                    Option<types::GetTemplateLanguagesVars>,
+                )>,
+                anyhow::Error,
+            > = Ok(Some((items, next_vars)));
+
+            res
+        },
+    )
+}
+
+/// Fetch all app templates by paginating through the responses.
+///
+/// Will fetch at most `max` templates.
+pub fn fetch_all_app_templates_from_framework(
+    client: &WasmerClient,
+    page_size: i32,
+    sort_by: Option<types::AppTemplatesSortBy>,
+    framework: String,
+) -> impl futures::Stream<Item = Result<Vec<types::AppTemplate>, anyhow::Error>> + '_ {
+    let vars = GetAppTemplatesFromFrameworkVars {
+        framework_slug: framework.clone().to_string(),
+        first: page_size,
+        sort_by,
+        after: None,
+    };
+
+    futures::stream::try_unfold(
+        Some(vars),
+        move |vars: Option<types::GetAppTemplatesFromFrameworkVars>| async move {
+            let vars = match vars {
+                Some(vars) => vars,
+                None => return Ok(None),
+            };
+
+            let con = client
+                .run_graphql_strict(types::GetAppTemplatesFromFramework::build(vars.clone()))
+                .await?
+                .get_app_templates
+                .context("backend did not return any data")?;
+
+            let items = con
+                .edges
+                .into_iter()
+                .flatten()
+                .filter_map(|edge| edge.node)
+                .collect::<Vec<_>>();
+
+            let next_cursor = con
+                .page_info
+                .end_cursor
+                .filter(|_| con.page_info.has_next_page);
+
+            let next_vars = next_cursor.map(|after| types::GetAppTemplatesFromFrameworkVars {
+                after: Some(after),
+                ..vars
+            });
+
+            #[allow(clippy::type_complexity)]
+            let res: Result<
+                Option<(
+                    Vec<types::AppTemplate>,
+                    Option<types::GetAppTemplatesFromFrameworkVars>,
+                )>,
+                anyhow::Error,
+            > = Ok(Some((items, next_vars)));
+
+            res
+        },
+    )
+}
+
+/// Fetch frameworks from available app templates.
+pub async fn fetch_app_template_frameworks(
+    client: &WasmerClient,
+    after: Option<String>,
+    first: Option<i32>,
+) -> Result<Option<types::TemplateFrameworkConnection>, anyhow::Error> {
+    client
+        .run_graphql_strict(types::GetTemplateFrameworks::build(
+            GetTemplateFrameworksVars { after, first },
+        ))
+        .await
+        .map(|r| r.get_template_frameworks)
+}
+
+/// Fetch all frameworks from available app templates by paginating through the responses.
+///
+/// Will fetch at most `max` templates.
+pub fn fetch_all_app_template_frameworks(
+    client: &WasmerClient,
+    page_size: Option<i32>,
+) -> impl futures::Stream<Item = Result<Vec<types::TemplateFramework>, anyhow::Error>> + '_ {
+    let vars = GetTemplateFrameworksVars {
+        after: None,
+        first: page_size,
+    };
+
+    futures::stream::try_unfold(
+        Some(vars),
+        move |vars: Option<types::GetTemplateFrameworksVars>| async move {
+            let vars = match vars {
+                Some(vars) => vars,
+                None => return Ok(None),
+            };
+
+            let con = client
+                .run_graphql_strict(types::GetTemplateFrameworks::build(vars.clone()))
+                .await?
+                .get_template_frameworks
+                .context("backend did not return any data")?;
+
+            let items = con
+                .edges
+                .into_iter()
+                .flatten()
+                .filter_map(|edge| edge.node)
+                .collect::<Vec<_>>();
+
+            let next_cursor = con
+                .page_info
+                .end_cursor
+                .filter(|_| con.page_info.has_next_page);
+
+            let next_vars = next_cursor.map(|after| types::GetTemplateFrameworksVars {
+                after: Some(after),
+                ..vars
+            });
+
+            #[allow(clippy::type_complexity)]
+            let res: Result<
+                Option<(
+                    Vec<types::TemplateFramework>,
+                    Option<types::GetTemplateFrameworksVars>,
+                )>,
+                anyhow::Error,
+            > = Ok(Some((items, next_vars)));
+
+            res
+        },
+    )
+}
+
 /// Get a signed URL to upload packages.
 pub async fn get_signed_url_for_package_upload(
     client: &WasmerClient,
@@ -193,7 +480,7 @@ pub async fn tag_package_release(
     homepage: Option<&str>,
     license: Option<&str>,
     license_file: Option<&str>,
-    manifest: &str,
+    manifest: Option<&str>,
     name: &str,
     namespace: Option<&str>,
     package_release_id: &cynic::Id,
@@ -429,6 +716,71 @@ pub async fn all_app_versions(
     Ok(all_versions)
 }
 
+/// Retrieve versions for an app.
+pub async fn get_deploy_app_versions_by_id(
+    client: &WasmerClient,
+    vars: types::GetDeployAppVersionsByIdVars,
+) -> Result<DeployAppVersionConnection, anyhow::Error> {
+    let res = client
+        .run_graphql_strict(types::GetDeployAppVersionsById::build(vars))
+        .await?;
+    let versions = res
+        .node
+        .context("app not found")?
+        .into_app()
+        .context("invalid node type returned")?
+        .versions;
+    Ok(versions)
+}
+
+/// Load all versions of an app id.
+///
+/// Will paginate through all versions and return them in a single list.
+pub async fn all_app_versions_by_id(
+    client: &WasmerClient,
+    app_id: impl Into<String>,
+) -> Result<Vec<DeployAppVersion>, anyhow::Error> {
+    let mut vars = types::GetDeployAppVersionsByIdVars {
+        id: cynic::Id::new(app_id),
+        offset: None,
+        before: None,
+        after: None,
+        first: Some(10),
+        last: None,
+        sort_by: None,
+    };
+
+    let mut all_versions = Vec::<DeployAppVersion>::new();
+
+    loop {
+        let page = get_deploy_app_versions_by_id(client, vars.clone()).await?;
+        if page.edges.is_empty() {
+            break;
+        }
+
+        for edge in page.edges {
+            let edge = match edge {
+                Some(edge) => edge,
+                None => continue,
+            };
+            let version = match edge.node {
+                Some(item) => item,
+                None => continue,
+            };
+
+            // Sanity check to avoid duplication.
+            if all_versions.iter().any(|v| v.id == version.id) == false {
+                all_versions.push(version);
+            }
+
+            // Update pagination.
+            vars.after = Some(edge.cursor);
+        }
+    }
+
+    Ok(all_versions)
+}
+
 /// Activate a particular version of an app.
 pub async fn app_version_activate(
     client: &WasmerClient,
