@@ -81,6 +81,7 @@ mod queries {
         pub size: Option<i32>,
         pub pirita_size: Option<i32>,
         pub webc_version: Option<WebcVersion>,
+        pub webc_manifest: Option<JSONString>,
     }
 
     #[derive(cynic::Enum, Clone, Copy, Debug)]
@@ -157,6 +158,46 @@ mod queries {
         Popular,
     }
 
+    #[derive(cynic::QueryVariables, Debug, Clone)]
+    pub struct GetAppTemplatesFromFrameworkVars {
+        pub framework_slug: String,
+        pub first: i32,
+        pub after: Option<String>,
+        pub sort_by: Option<AppTemplatesSortBy>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    #[cynic(graphql_type = "Query", variables = "GetAppTemplatesFromFrameworkVars")]
+    pub struct GetAppTemplatesFromFramework {
+        #[arguments(
+            frameworkSlug: $framework_slug,
+            first: $first,
+            after: $after,
+            sortBy: $sort_by
+        )]
+        pub get_app_templates: Option<AppTemplateConnection>,
+    }
+
+    #[derive(cynic::QueryVariables, Debug, Clone)]
+    pub struct GetAppTemplatesFromLanguageVars {
+        pub language_slug: String,
+        pub first: i32,
+        pub after: Option<String>,
+        pub sort_by: Option<AppTemplatesSortBy>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    #[cynic(graphql_type = "Query", variables = "GetAppTemplatesFromLanguageVars")]
+    pub struct GetAppTemplatesFromLanguage {
+        #[arguments(
+            languageSlug: $language_slug,
+            first: $first,
+            after: $after,
+            sortBy: $sort_by
+        )]
+        pub get_app_templates: Option<AppTemplateConnection>,
+    }
+
     #[derive(cynic::QueryVariables, Debug, Clone)]
     pub struct GetAppTemplatesVars {
         pub category_slug: String,
@@ -214,6 +255,80 @@ mod queries {
         pub use_cases: Jsonstring,
     }
 
+    #[derive(cynic::QueryVariables, Debug, Clone)]
+    pub struct GetTemplateFrameworksVars {
+        pub after: Option<String>,
+        pub first: Option<i32>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    #[cynic(graphql_type = "Query", variables = "GetTemplateFrameworksVars")]
+    pub struct GetTemplateFrameworks {
+        #[arguments(after: $after, first: $first)]
+        pub get_template_frameworks: Option<TemplateFrameworkConnection>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    pub struct TemplateFrameworkConnection {
+        pub edges: Vec<Option<TemplateFrameworkEdge>>,
+        pub page_info: PageInfo,
+        pub total_count: Option<i32>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    pub struct TemplateFrameworkEdge {
+        pub cursor: String,
+        pub node: Option<TemplateFramework>,
+    }
+
+    #[derive(serde::Serialize, cynic::QueryFragment, PartialEq, Eq, Debug)]
+    pub struct TemplateFramework {
+        #[serde(rename = "createdAt")]
+        pub created_at: DateTime,
+        pub id: cynic::Id,
+        pub name: String,
+        pub slug: String,
+        #[serde(rename = "updatedAt")]
+        pub updated_at: DateTime,
+    }
+
+    #[derive(cynic::QueryVariables, Debug, Clone)]
+    pub struct GetTemplateLanguagesVars {
+        pub after: Option<String>,
+        pub first: Option<i32>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    #[cynic(graphql_type = "Query", variables = "GetTemplateLanguagesVars")]
+    pub struct GetTemplateLanguages {
+        #[arguments(after: $after, first: $first)]
+        pub get_template_languages: Option<TemplateLanguageConnection>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    pub struct TemplateLanguageConnection {
+        pub edges: Vec<Option<TemplateLanguageEdge>>,
+        pub page_info: PageInfo,
+        pub total_count: Option<i32>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug)]
+    pub struct TemplateLanguageEdge {
+        pub cursor: String,
+        pub node: Option<TemplateLanguage>,
+    }
+
+    #[derive(serde::Serialize, cynic::QueryFragment, PartialEq, Eq, Debug)]
+    pub struct TemplateLanguage {
+        #[serde(rename = "createdAt")]
+        pub created_at: DateTime,
+        pub id: cynic::Id,
+        pub name: String,
+        pub slug: String,
+        #[serde(rename = "updatedAt")]
+        pub updated_at: DateTime,
+    }
+
     #[derive(cynic::Scalar, Debug, Clone, PartialEq, Eq)]
     #[cynic(graphql_type = "JSONString")]
     pub struct Jsonstring(pub String);
@@ -288,7 +403,7 @@ mod queries {
         pub homepage: Option<&'a str>,
         pub license: Option<&'a str>,
         pub license_file: Option<&'a str>,
-        pub manifest: &'a str,
+        pub manifest: Option<&'a str>,
         pub name: &'a str,
         pub namespace: Option<&'a str>,
         pub package_release_id: &'a cynic::Id,
@@ -765,6 +880,39 @@ mod queries {
         pub versions: DeployAppVersionConnection,
     }
 
+    #[derive(cynic::QueryVariables, Debug, Clone)]
+    pub struct GetDeployAppVersionsByIdVars {
+        pub id: cynic::Id,
+
+        pub offset: Option<i32>,
+        pub before: Option<String>,
+        pub after: Option<String>,
+        pub first: Option<i32>,
+        pub last: Option<i32>,
+        pub sort_by: Option<DeployAppVersionsSortBy>,
+    }
+
+    #[derive(cynic::QueryFragment, Debug, Clone, Serialize)]
+    #[cynic(graphql_type = "DeployApp", variables = "GetDeployAppVersionsByIdVars")]
+    pub struct DeployAppVersionsById {
+        #[arguments(
+            first: $first,
+            last: $last,
+            before: $before,
+            after: $after,
+            offset: $offset,
+            sortBy: $sort_by
+        )]
+        pub versions: DeployAppVersionConnection,
+    }
+
+    #[derive(cynic::QueryFragment, Debug, Clone)]
+    #[cynic(graphql_type = "Query", variables = "GetDeployAppVersionsByIdVars")]
+    pub struct GetDeployAppVersionsById {
+        #[arguments(id: $id)]
+        pub node: Option<NodeDeployAppVersions>,
+    }
+
     #[derive(cynic::QueryFragment, Serialize, Debug, Clone)]
     #[cynic(graphql_type = "DeployApp")]
     pub struct SparseDeployApp {
@@ -1595,6 +1743,23 @@ mod queries {
     #[derive(cynic::Scalar, Debug, Clone)]
     pub struct BigInt(pub i64);
 
+    #[derive(cynic::InlineFragments, Debug, Clone)]
+    #[cynic(graphql_type = "Node", variables = "GetDeployAppVersionsByIdVars")]
+    pub enum NodeDeployAppVersions {
+        DeployApp(Box<DeployAppVersionsById>),
+        #[cynic(fallback)]
+        Unknown,
+    }
+
+    impl NodeDeployAppVersions {
+        pub fn into_app(self) -> Option<DeployAppVersionsById> {
+            match self {
+                Self::DeployApp(v) => Some(*v),
+                _ => None,
+            }
+        }
+    }
+
     #[derive(cynic::InlineFragments, Debug)]
     pub enum Node {
         DeployApp(Box<DeployApp>),
