@@ -7,6 +7,7 @@ pub mod get;
 pub mod info;
 pub mod list;
 pub mod logs;
+pub mod purge_cache;
 pub mod version;
 
 mod util;
@@ -16,15 +17,16 @@ use crate::commands::AsyncCliCommand;
 /// Manage Wasmer Deploy apps.
 #[derive(clap::Subcommand, Debug)]
 pub enum CmdApp {
+    Deploy(deploy::CmdAppDeploy),
+    Create(create::CmdAppCreate),
     Get(get::CmdAppGet),
     Info(info::CmdAppInfo),
     List(list::CmdAppList),
     Logs(logs::CmdAppLogs),
-    Create(create::CmdAppCreate),
+    PurgeCache(purge_cache::CmdAppPurgeCache),
     Delete(delete::CmdAppDelete),
     #[clap(subcommand)]
     Version(version::CmdAppVersion),
-    Deploy(deploy::CmdAppDeploy),
 }
 
 #[async_trait::async_trait]
@@ -50,6 +52,7 @@ impl AsyncCliCommand for CmdApp {
             Self::Delete(cmd) => cmd.run_async().await,
             Self::Version(cmd) => cmd.run_async().await,
             Self::Deploy(cmd) => cmd.run_async().await,
+            Self::PurgeCache(cmd) => cmd.run_async().await,
         }
     }
 }
