@@ -95,6 +95,7 @@ impl PackagePublish {
                 env: self.env.clone(),
                 dry_run: self.dry_run,
                 quiet: self.quiet,
+                package_name: self.package_name.clone(),
                 package_namespace: self.package_namespace.clone(),
                 timeout: self.timeout,
                 non_interactive: self.non_interactive,
@@ -118,8 +119,15 @@ impl PackagePublish {
             non_interactive: self.non_interactive,
             package_path: self.package_path.clone(),
             package_hash,
+            package_id: None,
         }
-        .tag(client, manifest, manifest_path, true, allow_unnamed)
+        .tag(
+            client,
+            Some(manifest),
+            Some(manifest_path),
+            true,
+            allow_unnamed,
+        )
         .await
     }
 }
