@@ -7,6 +7,7 @@ use crate::ArtifactBuild;
 use crate::ArtifactBuildFromArchive;
 use crate::ArtifactCreate;
 use crate::Features;
+use crate::FrameInfosVariant;
 use crate::ModuleEnvironment;
 use crate::{
     register_frame_info, resolve_imports, FunctionExtent, GlobalFrameInfoRegistration,
@@ -35,8 +36,9 @@ use wasmer_types::DataInitializerLocation;
 use wasmer_types::DataInitializerLocationLike;
 use wasmer_types::MetadataHeader;
 use wasmer_types::{
-    CompileError, CpuFeature, DataInitializer, DeserializeError, FunctionIndex, LocalFunctionIndex,
-    MemoryIndex, ModuleInfo, OwnedDataInitializer, SignatureIndex, TableIndex, Target,
+    CompileError, CpuFeature, DataInitializer, DeserializeError, FunctionIndex, HashAlgorithm,
+    LocalFunctionIndex, MemoryIndex, ModuleInfo, OwnedDataInitializer, SignatureIndex, TableIndex,
+    Target,
 };
 use wasmer_types::{SerializableModule, SerializeError};
 use wasmer_vm::{FunctionBodyPtr, MemoryStyle, TableStyle, VMSharedSignatureIndex, VMTrampoline};
@@ -112,6 +114,7 @@ impl Artifact {
         engine: &Engine,
         data: &[u8],
         tunables: &dyn Tunables,
+        hash_algorithm: Option<HashAlgorithm>,
     ) -> Result<Self, CompileError> {
         let mut inner_engine = engine.inner_mut();
         let environ = ModuleEnvironment::new();
@@ -134,6 +137,7 @@ impl Artifact {
             engine.target(),
             memory_styles,
             table_styles,
+            hash_algorithm,
         )?;
 
         Self::from_parts(
@@ -680,8 +684,10 @@ impl Artifact {
             self.artifact.create_module_info(),
             &finished_function_extents,
             match &self.artifact {
-                ArtifactBuildVariant::Plain(p) => p.get_frame_info_ref().clone(),
-                ArtifactBuildVariant::Archived(a) => a.deserialize_frame_info_ref()?,
+                ArtifactBuildVariant::Plain(p) => {
+                    FrameInfosVariant::Owned(p.get_frame_info_ref().clone())
+                }
+                ArtifactBuildVariant::Archived(a) => FrameInfosVariant::Archived(a.clone()),
             },
         );
 
