@@ -11,15 +11,24 @@
 //! let module: ModuleInfo = ...;
 //! FRAME_INFO.register(module, compiled_functions);
 //! ```
+use core::ops::Deref;
+use rkyv::vec::ArchivedVec;
 use std::cmp;
 use std::collections::BTreeMap;
 use std::sync::{Arc, RwLock};
+use wasmer_types::compilation::address_map::{
+    ArchivedFunctionAddressMap, ArchivedInstructionAddressMap,
+};
+use wasmer_types::compilation::function::ArchivedCompiledFunctionFrameInfo;
 use wasmer_types::entity::{BoxedSlice, EntityRef, PrimaryMap};
 use wasmer_types::{
-    CompiledFunctionFrameInfo, FrameInfo, LocalFunctionIndex, ModuleInfo, TrapInformation,
+    CompiledFunctionFrameInfo, FrameInfo, FunctionAddressMap, InstructionAddressMap,
+    LocalFunctionIndex, ModuleInfo, SourceLoc, TrapInformation,
 };
 use wasmer_vm::FunctionBodyPtr;
 
+use crate::ArtifactBuildFromArchive;
+
 lazy_static::lazy_static! {
     /// This is a global cache of backtrace frame information for all active
     ///
@@ -56,11 +65,14 @@ struct ModuleInfoFrameInfo {
     start: usize,
     functions: BTreeMap<usize, FunctionInfo>,
     module: Arc<ModuleInfo>,
-    frame_infos: PrimaryMap<LocalFunctionIndex, CompiledFunctionFrameInfo>,
+    frame_infos: FrameInfosVariant,
 }
 
 impl ModuleInfoFrameInfo {
-    fn function_debug_info(&self, local_index: LocalFunctionIndex) -> &CompiledFunctionFrameInfo {
+    fn function_debug_info(
+        &self,
+        local_index: LocalFunctionIndex,
+    ) -> CompiledFunctionFrameInfoVariant {
         self.frame_infos.get(local_index).unwrap()
     }
 
@@ -94,11 +106,9 @@ impl GlobalFrameInfo {
         // machine instruction that corresponds to `pc`, which then allows us to
         // map that to a wasm original source location.
         let rel_pos = pc - func.start;
-        let instr_map = &module.function_debug_info(func.local_index).address_map;
-        let pos = match instr_map
-            .instructions
-            .binary_search_by_key(&rel_pos, |map| map.code_offset)
-        {
+        let debug_info = module.function_debug_info(func.local_index);
+        let instr_map = debug_info.address_map();
+        let pos = match instr_map.instructions().code_offset_by_key(rel_pos) {
             // Exact hit!
             Ok(pos) => Some(pos),
 
@@ -112,7 +122,7 @@ impl GlobalFrameInfo {
             // always get called with a `pc` that's an exact instruction
             // boundary.
             Err(n) => {
-                let instr = &instr_map.instructions[n - 1];
+                let instr = &instr_map.instructions().get(n - 1);
                 if instr.code_offset <= rel_pos && rel_pos < instr.code_offset + instr.code_len {
                     Some(n - 1)
                 } else {
@@ -122,32 +132,33 @@ impl GlobalFrameInfo {
         };
 
         let instr = match pos {
-            Some(pos) => instr_map.instructions[pos].srcloc,
+            Some(pos) => instr_map.instructions().get(pos).srcloc,
             // Some compilers don't emit yet the full trap information for each of
             // the instructions (such as LLVM).
             // In case no specific instruction is found, we return by default the
             // start offset of the function.
-            None => instr_map.start_srcloc,
+            None => instr_map.start_srcloc(),
         };
         let func_index = module.module.func_index(func.local_index);
         Some(FrameInfo::new(
             module.module.name(),
             func_index.index() as u32,
             module.module.function_names.get(&func_index).cloned(),
-            instr_map.start_srcloc,
+            instr_map.start_srcloc(),
             instr,
         ))
     }
 
     /// Fetches trap information about a program counter in a backtrace.
-    pub fn lookup_trap_info(&self, pc: usize) -> Option<&TrapInformation> {
+    pub fn lookup_trap_info(&self, pc: usize) -> Option<TrapInformation> {
         let module = self.module_info(pc)?;
         let func = module.function_info(pc)?;
-        let traps = &module.function_debug_info(func.local_index).traps;
+        let debug_info = module.function_debug_info(func.local_index);
+        let traps = debug_info.traps();
         let idx = traps
             .binary_search_by_key(&((pc - func.start) as u32), |info| info.code_offset)
             .ok()?;
-        Some(&traps[idx])
+        Some(traps[idx])
     }
 
     /// Gets a module given a pc
@@ -181,6 +192,162 @@ pub struct FunctionExtent {
     pub length: usize,
 }
 
+/// The variant of the frame information which can be an owned type
+/// or the explicit framed map
+#[derive(Debug)]
+pub enum FrameInfosVariant {
+    /// Owned frame infos
+    Owned(PrimaryMap<LocalFunctionIndex, CompiledFunctionFrameInfo>),
+    /// Archived frame infos
+    Archived(ArtifactBuildFromArchive),
+}
+
+impl FrameInfosVariant {
+    /// Gets the frame info for a given local function index
+    pub fn get(&self, index: LocalFunctionIndex) -> Option<CompiledFunctionFrameInfoVariant> {
+        match self {
+            Self::Owned(map) => map.get(index).map(CompiledFunctionFrameInfoVariant::Ref),
+            Self::Archived(archive) => archive
+                .get_frame_info_ref()
+                .get(index)
+                .map(CompiledFunctionFrameInfoVariant::Archived),
+        }
+    }
+}
+
+/// The variant of the compiled function frame info which can be an owned type
+#[derive(Debug)]
+pub enum CompiledFunctionFrameInfoVariant<'a> {
+    /// A reference to the frame info
+    Ref(&'a CompiledFunctionFrameInfo),
+    /// An archived frame info
+    Archived(&'a ArchivedCompiledFunctionFrameInfo),
+}
+
+impl CompiledFunctionFrameInfoVariant<'_> {
+    /// Gets the address map for the frame info
+    pub fn address_map(&self) -> FunctionAddressMapVariant<'_> {
+        match self {
+            CompiledFunctionFrameInfoVariant::Ref(info) => {
+                FunctionAddressMapVariant::Ref(&info.address_map)
+            }
+            CompiledFunctionFrameInfoVariant::Archived(info) => {
+                FunctionAddressMapVariant::Archived(&info.address_map)
+            }
+        }
+    }
+
+    /// Gets the traps for the frame info
+    pub fn traps(&self) -> VecTrapInformationVariant {
+        match self {
+            CompiledFunctionFrameInfoVariant::Ref(info) => {
+                VecTrapInformationVariant::Ref(&info.traps)
+            }
+            CompiledFunctionFrameInfoVariant::Archived(info) => {
+                VecTrapInformationVariant::Archived(&info.traps)
+            }
+        }
+    }
+}
+
+/// The variant of the trap information which can be an owned type
+#[derive(Debug)]
+pub enum VecTrapInformationVariant<'a> {
+    Ref(&'a Vec<TrapInformation>),
+    Archived(&'a ArchivedVec<TrapInformation>),
+}
+
+impl Deref for VecTrapInformationVariant<'_> {
+    type Target = [TrapInformation];
+
+    fn deref(&self) -> &Self::Target {
+        match self {
+            VecTrapInformationVariant::Ref(traps) => traps,
+            VecTrapInformationVariant::Archived(traps) => traps,
+        }
+    }
+}
+
+#[derive(Debug)]
+pub enum FunctionAddressMapVariant<'a> {
+    Ref(&'a FunctionAddressMap),
+    Archived(&'a ArchivedFunctionAddressMap),
+}
+
+impl FunctionAddressMapVariant<'_> {
+    pub fn instructions(&self) -> FunctionAddressMapInstructionVariant {
+        match self {
+            FunctionAddressMapVariant::Ref(map) => {
+                FunctionAddressMapInstructionVariant::Owned(&map.instructions)
+            }
+            FunctionAddressMapVariant::Archived(map) => {
+                FunctionAddressMapInstructionVariant::Archived(&map.instructions)
+            }
+        }
+    }
+
+    pub fn start_srcloc(&self) -> SourceLoc {
+        match self {
+            FunctionAddressMapVariant::Ref(map) => map.start_srcloc,
+            FunctionAddressMapVariant::Archived(map) => map.start_srcloc,
+        }
+    }
+
+    pub fn end_srcloc(&self) -> SourceLoc {
+        match self {
+            FunctionAddressMapVariant::Ref(map) => map.end_srcloc,
+            FunctionAddressMapVariant::Archived(map) => map.end_srcloc,
+        }
+    }
+
+    pub fn body_offset(&self) -> usize {
+        match self {
+            FunctionAddressMapVariant::Ref(map) => map.body_offset,
+            FunctionAddressMapVariant::Archived(map) => map.body_offset as usize,
+        }
+    }
+
+    pub fn body_len(&self) -> usize {
+        match self {
+            FunctionAddressMapVariant::Ref(map) => map.body_len,
+            FunctionAddressMapVariant::Archived(map) => map.body_len as usize,
+        }
+    }
+}
+
+#[derive(Debug)]
+pub enum FunctionAddressMapInstructionVariant<'a> {
+    Owned(&'a Vec<InstructionAddressMap>),
+    Archived(&'a ArchivedVec<ArchivedInstructionAddressMap>),
+}
+
+impl FunctionAddressMapInstructionVariant<'_> {
+    pub fn code_offset_by_key(&self, key: usize) -> Result<usize, usize> {
+        match self {
+            FunctionAddressMapInstructionVariant::Owned(instructions) => {
+                instructions.binary_search_by_key(&key, |map| map.code_offset)
+            }
+            FunctionAddressMapInstructionVariant::Archived(instructions) => {
+                instructions.binary_search_by_key(&key, |map| map.code_offset as usize)
+            }
+        }
+    }
+
+    pub fn get(&self, index: usize) -> InstructionAddressMap {
+        match self {
+            FunctionAddressMapInstructionVariant::Owned(instructions) => instructions[index],
+            FunctionAddressMapInstructionVariant::Archived(instructions) => instructions
+                .get(index)
+                .map(|map| InstructionAddressMap {
+                    srcloc: map.srcloc,
+                    code_offset: map.code_offset as usize,
+                    code_len: map.code_len as usize,
+                })
+                .unwrap(),
+        }
+    }
+}
+
 /// Registers a new compiled module's frame information.
 ///
 /// This function will register the `names` information for all of the
@@ -190,7 +357,7 @@ pub struct FunctionExtent {
 pub fn register(
     module: Arc<ModuleInfo>,
     finished_functions: &BoxedSlice<LocalFunctionIndex, FunctionExtent>,
-    frame_infos: PrimaryMap<LocalFunctionIndex, CompiledFunctionFrameInfo>,
+    frame_infos: FrameInfosVariant,
 ) -> Option<GlobalFrameInfoRegistration> {
     let mut min = usize::max_value();
     let mut max = 0;
