@@ -18,6 +18,10 @@ impl ArcFileSystem {
 }
 
 impl FileSystem for ArcFileSystem {
+    fn readlink(&self, path: &Path) -> Result<PathBuf> {
+        self.fs.readlink(path)
+    }
+
     fn read_dir(&self, path: &Path) -> Result<ReadDir> {
         self.fs.read_dir(path)
     }
