@@ -1,10 +1,11 @@
 // This file contains code from external sources.
-// Attributions: https://github.com/wasmerio/wasmer/blob/main/ATTRIBUTIONS.md
+// Attributions: https://github.com/wasmerio/wasmer/blob/main/docs/ATTRIBUTIONS.md
 
 //! Memory management for linear memories.
 //!
 //! `Memory` is to WebAssembly linear memories what `Table` is to WebAssembly tables.
 
+use crate::mmap::MmapType;
 use crate::threadconditions::ThreadConditions;
 pub use crate::threadconditions::{NotifyLocation, WaiterError};
 use crate::trap::Trap;
@@ -95,7 +96,8 @@ impl WasmMmap {
                     })?;
 
             let mut new_mmap =
-                Mmap::accessible_reserved(new_bytes, request_bytes).map_err(MemoryError::Region)?;
+                Mmap::accessible_reserved(new_bytes, request_bytes, None, MmapType::Private)
+                    .map_err(MemoryError::Region)?;
 
             let copy_len = self.alloc.len() - conf.offset_guard_size;
             new_mmap.as_mut_slice()[..copy_len].copy_from_slice(&self.alloc.as_slice()[..copy_len]);
@@ -207,7 +209,22 @@ impl VMOwnedMemory {
     /// This creates a `Memory` with owned metadata: this can be used to create a memory
     /// that will be imported into Wasm modules.
     pub fn new(memory: &MemoryType, style: &MemoryStyle) -> Result<Self, MemoryError> {
-        unsafe { Self::new_internal(memory, style, None) }
+        unsafe { Self::new_internal(memory, style, None, None, MmapType::Private) }
+    }
+
+    /// Create a new linear memory instance with specified minimum and maximum number of wasm pages
+    /// that is backed by a memory file. When set to private the file will be remaing in memory and
+    /// never flush to disk, when set to shared the memory will be flushed to disk.
+    ///
+    /// This creates a `Memory` with owned metadata: this can be used to create a memory
+    /// that will be imported into Wasm modules.
+    pub fn new_with_file(
+        memory: &MemoryType,
+        style: &MemoryStyle,
+        backing_file: std::path::PathBuf,
+        memory_type: MmapType,
+    ) -> Result<Self, MemoryError> {
+        unsafe { Self::new_internal(memory, style, None, Some(backing_file), memory_type) }
     }
 
     /// Create a new linear memory instance with specified minimum and maximum number of wasm pages.
@@ -222,7 +239,38 @@ impl VMOwnedMemory {
         style: &MemoryStyle,
         vm_memory_location: NonNull<VMMemoryDefinition>,
     ) -> Result<Self, MemoryError> {
-        Self::new_internal(memory, style, Some(vm_memory_location))
+        Self::new_internal(
+            memory,
+            style,
+            Some(vm_memory_location),
+            None,
+            MmapType::Private,
+        )
+    }
+
+    /// Create a new linear memory instance with specified minimum and maximum number of wasm pages
+    /// that is backed by a file. When set to private the file will be remaing in memory and
+    /// never flush to disk, when set to shared the memory will be flushed to disk.
+    ///
+    /// This creates a `Memory` with metadata owned by a VM, pointed to by
+    /// `vm_memory_location`: this can be used to create a local memory.
+    ///
+    /// # Safety
+    /// - `vm_memory_location` must point to a valid location in VM memory.
+    pub unsafe fn from_definition_with_file(
+        memory: &MemoryType,
+        style: &MemoryStyle,
+        vm_memory_location: NonNull<VMMemoryDefinition>,
+        backing_file: Option<std::path::PathBuf>,
+        memory_type: MmapType,
+    ) -> Result<Self, MemoryError> {
+        Self::new_internal(
+            memory,
+            style,
+            Some(vm_memory_location),
+            backing_file,
+            memory_type,
+        )
     }
 
     /// Build a `Memory` with either self-owned or VM owned metadata.
@@ -230,6 +278,8 @@ impl VMOwnedMemory {
         memory: &MemoryType,
         style: &MemoryStyle,
         vm_memory_location: Option<NonNull<VMMemoryDefinition>>,
+        backing_file: Option<std::path::PathBuf>,
+        memory_type: MmapType,
     ) -> Result<Self, MemoryError> {
         if memory.minimum > Pages::max_value() {
             return Err(MemoryError::MinimumMemoryTooLarge {
@@ -269,10 +319,16 @@ impl VMOwnedMemory {
         let mapped_pages = memory.minimum;
         let mapped_bytes = mapped_pages.bytes();
 
-        let mut alloc = Mmap::accessible_reserved(mapped_bytes.0, request_bytes)
-            .map_err(MemoryError::Region)?;
+        let mut alloc =
+            Mmap::accessible_reserved(mapped_bytes.0, request_bytes, backing_file, memory_type)
+                .map_err(MemoryError::Region)?;
+
         let base_ptr = alloc.as_mut_ptr();
-        let mem_length = memory.minimum.bytes().0;
+        let mem_length = memory
+            .minimum
+            .bytes()
+            .0
+            .max(alloc.as_slice_accessible().len());
         let mmap = WasmMmap {
             vm_memory_definition: if let Some(mem_loc) = vm_memory_location {
                 {
@@ -289,7 +345,7 @@ impl VMOwnedMemory {
                 })))
             },
             alloc,
-            size: memory.minimum,
+            size: Bytes::from(mem_length).try_into().unwrap(),
         };
 
         Ok(Self {
@@ -398,6 +454,21 @@ impl VMSharedMemory {
         Ok(VMOwnedMemory::new(memory, style)?.to_shared())
     }
 
+    /// Create a new linear memory instance with specified minimum and maximum number of wasm pages
+    /// that is backed by a file. When set to private the file will be remaing in memory and
+    /// never flush to disk, when set to shared the memory will be flushed to disk.
+    ///
+    /// This creates a `Memory` with owned metadata: this can be used to create a memory
+    /// that will be imported into Wasm modules.
+    pub fn new_with_file(
+        memory: &MemoryType,
+        style: &MemoryStyle,
+        backing_file: std::path::PathBuf,
+        memory_type: MmapType,
+    ) -> Result<Self, MemoryError> {
+        Ok(VMOwnedMemory::new_with_file(memory, style, backing_file, memory_type)?.to_shared())
+    }
+
     /// Create a new linear memory instance with specified minimum and maximum number of wasm pages.
     ///
     /// This creates a `Memory` with metadata owned by a VM, pointed to by
@@ -413,6 +484,32 @@ impl VMSharedMemory {
         Ok(VMOwnedMemory::from_definition(memory, style, vm_memory_location)?.to_shared())
     }
 
+    /// Create a new linear memory instance with specified minimum and maximum number of wasm pages
+    /// that is backed by a file. When set to private the file will be remaing in memory and
+    /// never flush to disk, when set to shared the memory will be flushed to disk.
+    ///
+    /// This creates a `Memory` with metadata owned by a VM, pointed to by
+    /// `vm_memory_location`: this can be used to create a local memory.
+    ///
+    /// # Safety
+    /// - `vm_memory_location` must point to a valid location in VM memory.
+    pub unsafe fn from_definition_with_file(
+        memory: &MemoryType,
+        style: &MemoryStyle,
+        vm_memory_location: NonNull<VMMemoryDefinition>,
+        backing_file: Option<std::path::PathBuf>,
+        memory_type: MmapType,
+    ) -> Result<Self, MemoryError> {
+        Ok(VMOwnedMemory::from_definition_with_file(
+            memory,
+            style,
+            vm_memory_location,
+            backing_file,
+            memory_type,
+        )?
+        .to_shared())
+    }
+
     /// Copies this memory to a new memory
     pub fn copy(&mut self) -> Result<Self, MemoryError> {
         let mut guard = self.mmap.write().unwrap();
