@@ -173,18 +173,29 @@ impl BinaryPackage {
         self.commands.iter().find(|cmd| cmd.name() == name)
     }
 
-    /// Get the bytes for the entrypoint command.
-    pub fn entrypoint_bytes(&self) -> Option<&[u8]> {
+    /// Resolve the entrypoint command name to a [`BinaryPackageCommand`].
+    pub fn get_entrypoint_command(&self) -> Option<&BinaryPackageCommand> {
         self.entrypoint_cmd
             .as_deref()
             .and_then(|name| self.get_command(name))
-            .map(|entry| entry.atom())
     }
 
+    /// Get the bytes for the entrypoint command.
+    #[deprecated(
+        note = "Use BinaryPackage::get_entrypoint_cmd instead",
+        since = "0.22.0"
+    )]
+    pub fn entrypoint_bytes(&self) -> Option<&[u8]> {
+        self.get_entrypoint_command().map(|entry| entry.atom())
+    }
+
+    /// Get a hash for this binary package.
+    ///
+    /// Usually the hash of the entrypoint.
     pub fn hash(&self) -> ModuleHash {
         *self.hash.get_or_init(|| {
-            if let Some(entry) = self.entrypoint_bytes() {
-                ModuleHash::xxhash(entry)
+            if let Some(cmd) = self.get_entrypoint_command() {
+                cmd.hash
             } else {
                 ModuleHash::xxhash(self.id.to_string())
             }
@@ -244,7 +255,7 @@ mod tests {
                 .with_shared_http_client(runtime.http_client().unwrap().clone()),
         );
 
-        let pkg = BinaryPackage::from_dir(&temp.path(), &runtime)
+        let pkg = BinaryPackage::from_dir(temp.path(), &runtime)
             .await
             .unwrap();
 
@@ -300,7 +311,7 @@ mod tests {
                 .with_shared_http_client(runtime.http_client().unwrap().clone()),
         );
 
-        let pkg = BinaryPackage::from_dir(&temp.path(), &runtime)
+        let pkg = BinaryPackage::from_dir(temp.path(), &runtime)
             .await
             .unwrap();
 
