@@ -18,7 +18,7 @@ use crate::{
 };
 use wasmer_types::ModuleHash;
 
-use super::wasi_common::MappedCommand;
+use super::wasi_common::{MappedCommand, MAPPED_CURRENT_DIR_DEFAULT_PATH};
 
 #[derive(Debug, Default, Clone)]
 pub struct WasiRunner {
@@ -80,6 +80,16 @@ impl WasiRunner {
         self.with_mounted_directories(dirs.into_iter().map(Into::into).map(MountedDirectory::from))
     }
 
+    pub fn with_home_mapped(&mut self, is_home_mapped: bool) -> &mut Self {
+        self.wasi.is_home_mapped = is_home_mapped;
+        self
+    }
+
+    pub fn with_tmp_mapped(&mut self, is_tmp_mapped: bool) -> &mut Self {
+        self.wasi.is_tmp_mapped = is_tmp_mapped;
+        self
+    }
+
     pub fn with_mounted_directories<I, D>(&mut self, dirs: I) -> &mut Self
     where
         I: IntoIterator<Item = D>,
@@ -250,6 +260,10 @@ impl WasiRunner {
             builder.set_stderr(Box::new(stderr.clone()));
         }
 
+        if self.wasi.is_home_mapped {
+            builder.set_current_dir(MAPPED_CURRENT_DIR_DEFAULT_PATH);
+        }
+
         Ok(builder)
     }
 
