@@ -16,7 +16,7 @@ pub fn fd_pipe<M: MemorySize>(
     ro_fd1: WasmPtr<WasiFd, M>,
     ro_fd2: WasmPtr<WasiFd, M>,
 ) -> Result<Errno, WasiError> {
-    let (fd1, fd2) = wasi_try_ok!(fd_pipe_internal(&mut ctx));
+    let (fd1, fd2) = wasi_try_ok!(fd_pipe_internal(&mut ctx, None, None));
     let env = ctx.data();
 
     #[cfg(feature = "journal")]
@@ -38,7 +38,11 @@ pub fn fd_pipe<M: MemorySize>(
     Ok(Errno::Success)
 }
 
-pub fn fd_pipe_internal(ctx: &mut FunctionEnvMut<'_, WasiEnv>) -> Result<(WasiFd, WasiFd), Errno> {
+pub fn fd_pipe_internal(
+    ctx: &mut FunctionEnvMut<'_, WasiEnv>,
+    with_fd1: Option<WasiFd>,
+    with_fd2: Option<WasiFd>,
+) -> Result<(WasiFd, WasiFd), Errno> {
     let env = ctx.data();
     let (memory, state, inodes) = unsafe { env.get_memory_and_wasi_state_and_inodes(&ctx, 0) };
     let (pipe1, pipe2) = Pipe::channel();
@@ -63,12 +67,28 @@ pub fn fd_pipe_internal(ctx: &mut FunctionEnvMut<'_, WasiEnv>) -> Result<(WasiFd
         | Rights::POLL_FD_READWRITE
         | Rights::SOCK_SEND
         | Rights::FD_FDSTAT_SET_FLAGS;
-    let fd1 = state
-        .fs
-        .create_fd(rights, rights, Fdflags::empty(), 0, inode1)?;
-    let fd2 = state
-        .fs
-        .create_fd(rights, rights, Fdflags::empty(), 0, inode2)?;
+
+    let fd1 = if let Some(fd) = with_fd1 {
+        state
+            .fs
+            .with_fd(rights, rights, Fdflags::empty(), 0, inode1, fd)
+            .map(|_| fd)?
+    } else {
+        state
+            .fs
+            .create_fd(rights, rights, Fdflags::empty(), 0, inode1)?
+    };
+
+    let fd2 = if let Some(fd) = with_fd2 {
+        state
+            .fs
+            .with_fd(rights, rights, Fdflags::empty(), 0, inode2, fd)
+            .map(|_| fd)?
+    } else {
+        state
+            .fs
+            .create_fd(rights, rights, Fdflags::empty(), 0, inode2)?
+    };
 
     Ok((fd1, fd2))
 }
