@@ -12,6 +12,6 @@ use crate::syscalls::*;
 /// * `rval` - The exit code returned by the process.
 #[instrument(level = "debug", skip_all, fields(%exitcode), ret)]
 pub fn thread_exit(ctx: FunctionEnvMut<'_, WasiEnv>, exitcode: ExitCode) -> Result<(), WasiError> {
-    tracing::debug!(%exitcode);
+    tracing::debug!(tid=%ctx.data().thread.id(), %exitcode);
     Err(WasiError::Exit(exitcode))
 }
